@@ -7,11 +7,9 @@
 
 /* hid_t H5Acreate( hid_t loc_id, const char *attr_name, hid_t type_id, hid_t space_id, hid_t acpl_id, hid_t aapl_id ) */
 SEXP _H5Acreate( SEXP _obj_id, SEXP _attr_name, SEXP _type_id, SEXP _space_id ) {
-  //hid_t obj_id = INTEGER(_obj_id)[0];
+  
   hid_t obj_id = STRSXP_2_HID( _obj_id );
   const char *attr_name = CHAR(STRING_ELT(_attr_name, 0));
-  //hid_t type_id = INTEGER(_type_id)[0];
-  //hid_t space_id =  INTEGER(_space_id)[0];
   hid_t type_id = STRSXP_2_HID( _type_id );
   hid_t space_id = STRSXP_2_HID( _space_id );
 
@@ -20,8 +18,6 @@ SEXP _H5Acreate( SEXP _obj_id, SEXP _attr_name, SEXP _type_id, SEXP _space_id )
   addHandle(hid);
 
   SEXP Rval;
-  //PROTECT(Rval = allocVector(INTSXP, 1));
-  //INTEGER(Rval)[0] = hid;
   PROTECT(Rval = HID_2_STRSXP(hid));
   UNPROTECT(1);
   return Rval;
@@ -45,7 +41,6 @@ SEXP _H5Aopen( SEXP _obj_id, SEXP _attr_name ) {
 
 /* hid_t H5Aopen_by_name( hid_t loc_id, const char *obj_name, const char *attr_name, hid_t aapl_id, hid_t lapl_id ) */
 SEXP _H5Aopen_by_name( SEXP _obj_id, SEXP _obj_name, SEXP _attr_name ) {
-  //hid_t obj_id = INTEGER(_obj_id)[0];
   hid_t obj_id = STRSXP_2_HID( _obj_id );
   const char *obj_name = CHAR(STRING_ELT(_obj_name, 0));
   const char *attr_name = CHAR(STRING_ELT(_attr_name, 0));
@@ -53,8 +48,6 @@ SEXP _H5Aopen_by_name( SEXP _obj_id, SEXP _obj_name, SEXP _attr_name ) {
   addHandle( hid );
 
   SEXP Rval;
-  //PROTECT(Rval = allocVector(INTSXP, 1));
-  //INTEGER(Rval)[0] = hid;
   PROTECT(Rval = HID_2_STRSXP(hid));
   UNPROTECT(1);
   return Rval;
@@ -62,7 +55,6 @@ SEXP _H5Aopen_by_name( SEXP _obj_id, SEXP _obj_name, SEXP _attr_name ) {
 
 /* hid_t H5Aopen_by_idx( hid_t loc_id, const char *obj_name, H5_index_t idx_type, H5_iter_order_t order, hsize_t n, hid_t aapl_id, hid_t lapl_id ) */
 SEXP _H5Aopen_by_idx( SEXP _obj_id, SEXP _obj_name, SEXP _idx_type, SEXP _order, SEXP _n ) {
-  //hid_t obj_id = INTEGER(_obj_id)[0];
   hid_t obj_id = STRSXP_2_HID( _obj_id );
   const char *obj_name = CHAR(STRING_ELT(_obj_name, 0));
   H5_index_t idx_type = INTEGER(_idx_type)[0];
@@ -72,8 +64,6 @@ SEXP _H5Aopen_by_idx( SEXP _obj_id, SEXP _obj_name, SEXP _idx_type, SEXP _order,
   addHandle( hid );
 
   SEXP Rval;
-  //PROTECT(Rval = allocVector(INTSXP, 1));
-  //INTEGER(Rval)[0] = hid;
   PROTECT(Rval = HID_2_STRSXP(hid));
   UNPROTECT(1);
   return Rval;
@@ -107,7 +97,6 @@ SEXP _H5Aclose( SEXP _attr_id ) {
 
 /* herr_t H5Adelete( hid_t loc_id, const char *attr_name ) */
 SEXP _H5Adelete( SEXP _obj_id, SEXP _attr_name ) {
-  //hid_t obj_id = INTEGER(_obj_id)[0];
   hid_t obj_id = STRSXP_2_HID( _obj_id );
   const char *attr_name = CHAR(STRING_ELT(_attr_name, 0));
   herr_t herr = H5Adelete( obj_id, attr_name );
@@ -116,33 +105,102 @@ SEXP _H5Adelete( SEXP _obj_id, SEXP _attr_name ) {
 }
 
 
-SEXP H5Aread_helper_INTEGER(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t dtype_id) {
+SEXP H5Aread_helper_INTEGER(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t dtype_id,
+                            int bit64conversion) {
+    
   hid_t mem_type_id = -1;
 
   SEXP Rval;
-  /* if (cpdType < 0) { */
-    mem_type_id = H5T_NATIVE_INT;
-  /* } else { */
-  /*   mem_type_id = H5Tcreate(H5T_COMPOUND, H5Tget_size(H5T_NATIVE_INT)); */
-  /*   herr_t status = H5Tinsert(mem_type_id, cpdField[0], 0, H5T_NATIVE_INT); */
-  /*   for (int i=1; i<cpdNField; i++) { */
-  /*     hid_t mem_type_id2 = H5Tcreate(H5T_COMPOUND, H5Tget_size(H5T_NATIVE_INT)); */
-  /*     herr_t status = H5Tinsert(mem_type_id2, cpdField[i], 0, mem_type_id); */
-  /*     mem_type_id = mem_type_id2; */
-  /*   } */
-  /* } */
-  void * buf;
-  if (length(_buf) == 0) {
-    Rval = PROTECT(allocVector(INTSXP, n));
-    buf = INTEGER(Rval);
-  } else {
-    buf = INTEGER(_buf);
-    Rval = _buf;
-  }
-  herr_t herr = H5Aread(attr_id, mem_type_id, buf );
-  if (length(_buf) == 0) {
-    setAttrib(Rval, R_DimSymbol, Rdim);
-    UNPROTECT(1);
+  int b = H5Tget_size(dtype_id);
+  H5T_sign_t sgn = H5Tget_sign(dtype_id);
+  herr_t herr;
+  
+  if((b < 4) | ((b == 4) & (sgn == H5T_SGN_2))) {
+      mem_type_id = H5T_NATIVE_INT;
+    
+      void * buf;
+      if (length(_buf) == 0) {
+        Rval = PROTECT(allocVector(INTSXP, n));
+        buf = INTEGER(Rval);
+      } else {
+        buf = INTEGER(_buf);
+        Rval = _buf;
+      }
+      herr = H5Aread(attr_id, mem_type_id, buf );
+      if (length(_buf) == 0) {
+        setAttrib(Rval, R_DimSymbol, Rdim);
+        UNPROTECT(1);
+      }
+  } else if ( ((b == 4) & (sgn == H5T_SGN_NONE)) | (b == 8) ) { 
+      // unsigned32-bit or 64-bit integer
+      void* intbuf;
+      void* buf;
+      
+      if(b == 4) {
+          mem_type_id = H5T_STD_U32LE;
+          intbuf = R_alloc(n, sizeof(unsigned int));
+      } else if((b == 8) & (sgn == H5T_SGN_NONE)) {
+          mem_type_id = H5T_NATIVE_UINT64;
+          intbuf = R_alloc(n, sizeof(unsigned long long));
+      } else {
+          mem_type_id = H5T_NATIVE_INT64;
+          intbuf = R_alloc(n, sizeof(long long));
+      }
+      
+      if (intbuf == 0) {
+          error("Not enough memory to read the attribute.");
+      }
+      
+      herr = H5Aread(attr_id, mem_type_id, intbuf );
+      if(herr< 0) {
+        error("Error reading attribute");
+      }
+      
+      if (bit64conversion == 0) {  // Convert data to R-integer and replace overflow values with NA_integer
+
+          if (length(_buf) == 0) {
+              Rval = PROTECT(allocVector(INTSXP, n));
+              buf = (int *) INTEGER(Rval);
+          } else {
+              buf = INTEGER(_buf);
+              Rval = _buf;
+          }
+          if ((b == 4) & (sgn == H5T_SGN_NONE)) {
+              uint32_to_int32(intbuf, n, buf);
+          } else if (b == 8) { 
+              int64_to_int32(intbuf, n, buf, sgn);
+          }
+      } else {
+
+          if (length(_buf) == 0) {
+              Rval = PROTECT(allocVector(REALSXP, n));
+              buf = (long long *) REAL(Rval);
+          } else {
+              buf = REAL(_buf);
+              Rval = _buf;
+          }
+          if (bit64conversion == 1) {  //convert to double
+              if ((b == 4) & (sgn == H5T_SGN_NONE)) {
+                  uint32_to_double(intbuf, n, buf);
+              } else if (b == 8) {
+                  int64_to_double(intbuf, n, buf, sgn);
+              }
+          } else { // convert to integer64
+              if((b == 4) & (sgn == H5T_SGN_NONE)) {
+                  uint32_to_integer64(intbuf, n, buf);
+              } else if (b == 8) {
+                  int64_to_integer64(intbuf, n, buf, sgn);
+              }
+              SEXP la = PROTECT(mkString("integer64"));
+              setAttrib(Rval, R_ClassSymbol, la);
+              UNPROTECT(1);
+          }
+      }
+      
+      if (length(_buf) == 0) {
+          setAttrib(Rval, R_DimSymbol, Rdim);
+          UNPROTECT(1);
+      }
   }
 
   return(Rval);
@@ -153,17 +211,7 @@ SEXP H5Aread_helper_FLOAT(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t
   hid_t mem_type_id = -1;
 
   SEXP Rval;
-  /* if (cpdType < 0) { */
-    mem_type_id = H5T_NATIVE_DOUBLE;
-  /* } else { */
-  /*   mem_type_id = H5Tcreate(H5T_COMPOUND, H5Tget_size(H5T_NATIVE_DOUBLE)); */
-  /*   herr_t status = H5Tinsert(mem_type_id, cpdField[0], 0, H5T_NATIVE_DOUBLE); */
-  /*   for (int i=1; i<cpdNField; i++) { */
-  /*     hid_t mem_type_id2 = H5Tcreate(H5T_COMPOUND, H5Tget_size(H5T_NATIVE_DOUBLE)); */
-  /*     herr_t status = H5Tinsert(mem_type_id2, cpdField[i], 0, mem_type_id); */
-  /*     mem_type_id = mem_type_id2; */
-  /*   } */
-  /* } */
+  mem_type_id = H5T_NATIVE_DOUBLE;
   void * buf;
   if (length(_buf) == 0) {
     Rval = PROTECT(allocVector(REALSXP, n));
@@ -172,7 +220,12 @@ SEXP H5Aread_helper_FLOAT(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t
     buf = REAL(_buf);
     Rval = _buf;
   }
+  
   herr_t herr = H5Aread(attr_id, mem_type_id, buf );
+  if(herr < 0) {
+    error("Error reading attribute");
+  }
+  
   if (length(_buf) == 0) {
     setAttrib(Rval, R_DimSymbol, Rdim);
     UNPROTECT(1);
@@ -185,17 +238,7 @@ SEXP H5Aread_helper_STRING(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t
 
   SEXP Rval;
   size_t size = H5Tget_size(dtype_id);
-  /* if (cpdType < 0) { */
-    mem_type_id = dtype_id;
-  /* } else { */
-  /*   mem_type_id = H5Tcreate(H5T_COMPOUND, size); */
-  /*   herr_t status = H5Tinsert(mem_type_id, cpdField[0], 0, dtype_id); */
-  /*   for (int i=1; i<cpdNField; i++) { */
-  /* 	hid_t mem_type_id2 = H5Tcreate(H5T_COMPOUND, size); */
-  /* 	herr_t status = H5Tinsert(mem_type_id2, cpdField[i], 0, mem_type_id); */
-  /* 	mem_type_id = mem_type_id2; */
-  /*   } */
-  /* } */
+  mem_type_id = dtype_id;
   Rval = PROTECT(allocVector(STRSXP, n));
   if (H5Tis_variable_str(dtype_id)) {
     char *bufSTR[n];
@@ -206,7 +249,12 @@ SEXP H5Aread_helper_STRING(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t
     }
   } else {
     char bufSTR[n][size];
+    
     herr_t herr = H5Aread(attr_id, mem_type_id, bufSTR );
+    if(herr < 0) {
+      error("Error reading attribute");
+    }
+    
     char bufSTR2[n][size+1];
     for (int i=0; i<n; i++) {
       for (int j=0; j<size; j++) {
@@ -287,16 +335,16 @@ SEXP H5Aread_helper_STRING(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t
 /*   return(Rval); */
 /* } */
 
-SEXP H5Aread_helper(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf ) {
-
-  hid_t dtype_id;
-  dtype_id = H5Aget_type(attr_id);
-  hid_t dtypeclass_id = H5Tget_class(dtype_id);
+SEXP H5Aread_helper(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, int bit64conversion ) {
+    
+    hid_t dtype_id;
+    dtype_id = H5Aget_type(attr_id);
+    hid_t dtypeclass_id = H5Tget_class(dtype_id);
 
   SEXP Rval;
   switch(dtypeclass_id) {
   case H5T_INTEGER: {
-    Rval = H5Aread_helper_INTEGER(attr_id, n, Rdim, _buf, dtype_id);
+    Rval = H5Aread_helper_INTEGER(attr_id, n, Rdim, _buf, dtype_id, bit64conversion);
   } break;
   case H5T_FLOAT: {
     Rval = H5Aread_helper_FLOAT(attr_id, n, Rdim, _buf, dtype_id);
@@ -331,12 +379,13 @@ SEXP H5Aread_helper(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf ) {
 }
 
 /* herr_t H5Aread(hid_t attr_id, hid_t mem_type_id, void *buf ) */
-SEXP _H5Aread( SEXP _attr_id, SEXP _buf ) {
+SEXP _H5Aread( SEXP _attr_id, SEXP _buf, SEXP _bit64conversion ) {
+    
+  int bit64conversion = INTEGER(_bit64conversion)[0];
 
   /***********************************************************************/
   /* attr_id                                                          */
   /***********************************************************************/
-  //hid_t attr_id = INTEGER(_attr_id)[0];
   hid_t attr_id = STRSXP_2_HID( _attr_id );
   /***********************************************************************/
   /* file_space_id and get dimensionality of output file_space and buf   */
@@ -374,7 +423,7 @@ SEXP _H5Aread( SEXP _attr_id, SEXP _buf ) {
   /* read file space data type                                           */
   /***********************************************************************/
 
-  SEXP Rval = H5Aread_helper(attr_id, n, Rdim, _buf);
+  SEXP Rval = H5Aread_helper(attr_id, n, Rdim, _buf, bit64conversion);
 
   // close mem space
   H5Sclose(mem_space_id);
@@ -389,7 +438,6 @@ SEXP _H5Aread( SEXP _attr_id, SEXP _buf ) {
 
 /* herr_t H5Awrite(hid_t attr_id, hid_t mem_type_id, const void *buf ) */
 SEXP _H5Awrite( SEXP _attr_id, SEXP _buf) {
-  //hid_t attr_id = INTEGER(_attr_id)[0];
   hid_t attr_id = STRSXP_2_HID( _attr_id );
   hid_t mem_type_id;
 
@@ -403,20 +451,29 @@ SEXP _H5Awrite( SEXP _attr_id, SEXP _buf) {
       buf = REAL(_buf);
     } else {
       if (TYPEOF(_buf) == STRSXP) {
-	mem_type_id = H5Aget_type(attr_id);
-	size_t stsize = H5Tget_size( mem_type_id );
-	char * strbuf = (char *)R_alloc(LENGTH(_buf),stsize);
-	int z=0;
-	int j;
-	for (int i=0; i < LENGTH(_buf); i++) {
-	  for (j=0; (j < LENGTH(STRING_ELT(_buf,i))) & (j < (stsize-1)); j++) {
-	    strbuf[z++] = CHAR(STRING_ELT(_buf,i))[j];
-	  }
-	  for (; j < stsize; j++) {
-	    strbuf[z++] = '\0';
-	  }
-	}
-	buf = strbuf;
+        mem_type_id = H5Aget_type(attr_id);
+        size_t stsize;
+        if (H5Tis_variable_str(mem_type_id)) {
+          const char ** strbuf = (const char **)R_alloc(LENGTH(_buf),sizeof(const char*));
+          for (int i=0; i < LENGTH(_buf); i++) {
+            strbuf[i] = CHAR(STRING_ELT(_buf,i));
+          }
+          buf = strbuf;
+        } else {
+          stsize = H5Tget_size( mem_type_id );
+          char * strbuf = (char *)R_alloc(LENGTH(_buf),stsize);
+          int z=0;
+          int j;
+          for (int i=0; i < LENGTH(_buf); i++) {
+            for (j=0; (j < LENGTH(STRING_ELT(_buf,i))) & (j < (stsize-1)); j++) {
+              strbuf[z++] = CHAR(STRING_ELT(_buf,i))[j];
+            }
+            for (; j < stsize; j++) {
+              strbuf[z++] = '\0';
+            }
+          }
+          buf = strbuf;
+        }
       } else {
 	mem_type_id = -1;
 	warning("Writing of this type of attribute data not supported.");
