@@ -164,6 +164,10 @@ SEXP H5Dread_helper_INTEGER(hid_t dataset_id, hid_t file_space_id, hid_t mem_spa
           Rval = _buf;
         }
         herr_t herr = H5Dread(dataset_id, mem_type_id, mem_space_id, file_space_id, H5P_DEFAULT, buf );
+        if(herr < 0) {
+          error("Error reading dataset");
+        }
+        
         if (native)
           PERMUTE(Rval, RAW, mem_space_id);
         
@@ -188,6 +192,10 @@ SEXP H5Dread_helper_INTEGER(hid_t dataset_id, hid_t file_space_id, hid_t mem_spa
           Rval = _buf;
         }
         herr_t herr = H5Dread(dataset_id, mem_type_id, mem_space_id, file_space_id, H5P_DEFAULT, buf );
+        if(herr < 0) {
+          error("Error reading dataset");
+        }
+        
         if (native)
           PERMUTE(Rval, INTEGER, mem_space_id);
         
@@ -218,7 +226,10 @@ SEXP H5Dread_helper_INTEGER(hid_t dataset_id, hid_t file_space_id, hid_t mem_spa
             Rval = _buf;
         }
         herr_t herr = H5Dread(dataset_id, mem_type_id, mem_space_id, file_space_id, H5P_DEFAULT, buf );
-
+        if(herr < 0) {
+          error("Error reading dataset");
+        }
+        
         if (native)
             PERMUTE(Rval, INTEGER, mem_space_id);
         
@@ -257,6 +268,9 @@ SEXP H5Dread_helper_INTEGER(hid_t dataset_id, hid_t file_space_id, hid_t mem_spa
         }
         
         herr_t herr = H5Dread(dataset_id, mem_type_id, mem_space_id, file_space_id, H5P_DEFAULT, intbuf );
+        if(herr < 0) {
+          error("Error reading dataset");
+        }
         
         if (bit64conversion == 0) {  // Convert data to R-integer and replace overflow values with NA_integer
             void * buf;
@@ -267,41 +281,10 @@ SEXP H5Dread_helper_INTEGER(hid_t dataset_id, hid_t file_space_id, hid_t mem_spa
                 buf = INTEGER(_buf);
                 Rval = _buf;
             }
-            long long i;
             if ((b == 4) & (sgn == H5T_SGN_NONE)) {
-                for (i=0; i<n; i++) {
-                    ((int *)buf)[i] = ((unsigned int *)intbuf)[i];
-                }
-                for (i=0; i<n; i++) {
-                    if (((unsigned int *)intbuf)[i] > INT_MAX) {
-                        ((int *)buf)[i] = INT_MIN;
-                        warn = 1;
-                    }
-                }
-            } else if ((b == 8) & (sgn == H5T_SGN_2)) {
-                for (i=0; i<n; i++) {
-                    ((int *)buf)[i] = ((long long *)intbuf)[i];
-                }
-                for (i=0; i<n; i++) {
-                    if (((long long *)intbuf)[i] > INT_MAX) {
-                        ((int *)buf)[i] = INT_MIN;
-                        warn = 1;
-                    }
-                    if (((long long *)intbuf)[i] < INT_MIN) {
-                        ((int *)buf)[i] = INT_MIN;
-                        warn = 1;
-                    }
-                }
-            } else if ((b == 8) & (sgn == H5T_SGN_NONE)) {
-                for (i=0; i<n; i++) {
-                    ((int *)buf)[i] = ((unsigned long long *)intbuf)[i];
-                }
-                for (i=0; i<n; i++) {
-                    if (((unsigned long long *)intbuf)[i] > INT_MAX) {
-                        ((int *)buf)[i] = INT_MIN;
-                        warn = 1;
-                    }
-                }
+                uint32_to_int32(intbuf, n, buf);
+            } else if (b == 8) { 
+                int64_to_int32(intbuf, n, buf, sgn);
             }
         } else {
             void * buf;
@@ -319,30 +302,9 @@ SEXP H5Dread_helper_INTEGER(hid_t dataset_id, hid_t file_space_id, hid_t mem_spa
                         ((double *)buf)[i] = ((int *)intbuf)[i];
                     }
                 } else if ((b == 4) & (sgn == H5T_SGN_NONE)) {
-                    for (i=0; i<n; i++){
-                        ((double *)buf)[i] = ((unsigned int *)intbuf)[i];
-                    }
-                } else if ((b == 8) & (sgn == H5T_SGN_2)) {
-                    for (i=0; i<n; i++){
-                        ((double *)buf)[i] = ((long long *)intbuf)[i];
-                    }
-                    for (i=0; i<n; i++) {
-                        if (((long long *)intbuf)[i] > 0x001fffffffffffffL) {
-                            warn_double = 1;
-                        }
-                        if (((long long *)intbuf)[i] < (long long) 0xffe0000000000000L) {
-                            warn_double = 1;
-                        }
-                    }
-                } else if ((b == 8) & (sgn == H5T_SGN_NONE)) {
-                    for (i=0; i<n; i++){
-                        ((double *)buf)[i] = ((unsigned long long *)intbuf)[i];
-                    }
-                    for (i=0; i<n; i++) {
-                        if (((unsigned long long *)intbuf)[i] > 0x001fffffffffffffUL) {
-                            warn_double = 1;
-                        }
-                    }
+                    uint32_to_double(intbuf, n, buf);
+                } else if (b == 8) {
+                    int64_to_double(intbuf, n, buf, sgn);
                 }
             } else { // convert to integer64 class
                 long long i;
@@ -351,23 +313,9 @@ SEXP H5Dread_helper_INTEGER(hid_t dataset_id, hid_t file_space_id, hid_t mem_spa
                         ((long long *)buf)[i] = ((int *)intbuf)[i];
                     }
                 } else if ((b == 4) & (sgn == H5T_SGN_NONE)) {
-                    for (i=0; i<n; i++){
-                        ((long long *)buf)[i] = ((unsigned int *)intbuf)[i];
-                    }
-                } else if ((b == 8) & (sgn == H5T_SGN_2)) {
-                    for (i=0; i<n; i++){
-                        ((long long *)buf)[i] = ((long long *)intbuf)[i];
-                    }
-                } else if ((b == 8) & (sgn == H5T_SGN_NONE)) {
-                    for (i=0; i<n; i++){
-                        ((long long *)buf)[i] = ((unsigned long long *)intbuf)[i];
-                    }
-                    for (i=0; i<n; i++) {
-                        if (((unsigned long long *)intbuf)[i] > LLONG_MAX) {
-                            ((long long *)buf)[i] = LLONG_MIN;
-                            warn_overflow_64bit = 1;
-                        }
-                    }
+                    uint32_to_integer64(intbuf, n, buf);
+                } else if (b == 8) {
+                    int64_to_integer64(intbuf, n, buf, sgn);
                 }
                 if (native)
                     PERMUTE(Rval, INTEGER, mem_space_id);
@@ -1005,20 +953,30 @@ SEXP _H5Dwrite( SEXP _dataset_id, SEXP _buf, SEXP _file_space_id, SEXP _mem_spac
         mem_type_id = H5Dget_type(dataset_id);
         if (native)
             PERMUTE(_buf, STRING_PTR, dim_space_id);
+
         /* prepare for hdf5 */
-        size_t stsize = H5Tget_size( mem_type_id );
-        char * strbuf = (char *)R_alloc(LENGTH(_buf),stsize);
-        int z=0;
-        int j;
-        for (int i=0; i < LENGTH(_buf); i++) {
-            for (j=0; (j < LENGTH(STRING_ELT(_buf,i))) & (j < stsize); j++) {
-                strbuf[z++] = CHAR(STRING_ELT(_buf,i))[j];
+        if (!H5Tis_variable_str(mem_type_id)) {
+            size_t stsize = H5Tget_size( mem_type_id );
+            char * strbuf = (char *)R_alloc(LENGTH(_buf),stsize);
+            int z=0;
+            int j;
+            for (int i=0; i < LENGTH(_buf); i++) {
+                for (j=0; (j < LENGTH(STRING_ELT(_buf,i))) & (j < stsize); j++) {
+                    strbuf[z++] = CHAR(STRING_ELT(_buf,i))[j];
+                }
+                for (; j < stsize; j++) {
+                    strbuf[z++] = '\0';
+                }
             }
-            for (; j < stsize; j++) {
-                strbuf[z++] = '\0';
+            buf = strbuf;
+        } else {
+            const char ** strbuf = (const char **)R_alloc(LENGTH(_buf), sizeof(char*));
+            for (int i=0; i < LENGTH(_buf); i++) {
+                strbuf[i] = CHAR(STRING_ELT(_buf, i));
             }
+            buf = strbuf;
         }
-        buf = strbuf;
+
         break;
     default :
         mem_type_id = -1;
@@ -1092,4 +1050,3 @@ SEXP _H5Dset_extent( SEXP _dataset_id, SEXP _size ) {
     UNPROTECT(1);
     return Rval;
 }
-
