@@ -13,5 +13,8 @@ SEXP _H5Tset_size( SEXP _dtype_id, SEXP _size );
 SEXP _H5Tget_size( SEXP _dtype_id );
 SEXP _H5Tset_strpad( SEXP _dtype_id, SEXP _strpad );
 SEXP _H5Tget_strpad( SEXP _dtype_id );
+SEXP _H5Tset_cset( SEXP _dtype_id, SEXP _cset );
+SEXP _H5Tget_cset( SEXP _dtype_id );
+SEXP _H5Tis_variable_str( SEXP _dtype_id );
 
 #endif
