@@ -41,7 +41,7 @@ static R_CallMethodDef libraryRCalls[] = {
   {"_H5Aopen_by_name", (DL_FUNC) &_H5Aopen_by_name, 3},
   {"_H5Aopen_by_idx", (DL_FUNC) &_H5Aopen_by_idx, 5},
   {"_H5Aexists", (DL_FUNC) &_H5Aexists, 2},
-  {"_H5Aread", (DL_FUNC) &_H5Aread, 2},
+  {"_H5Aread", (DL_FUNC) &_H5Aread, 3},
   {"_H5Awrite", (DL_FUNC) &_H5Awrite, 2},
   {"_H5Aclose", (DL_FUNC) &_H5Aclose, 1},
   {"_H5Adelete", (DL_FUNC) &_H5Adelete, 2},
@@ -82,6 +82,7 @@ static R_CallMethodDef libraryRCalls[] = {
   {"_H5Iis_valid", (DL_FUNC) &_H5Iis_valid, 1},
   {"_H5Oopen", (DL_FUNC) &_H5Oopen, 2},
   {"_H5Oclose", (DL_FUNC) &_H5Oclose, 1},
+  {"_H5Olink", (DL_FUNC) &_H5Olink, 5},
   {"_H5Oget_num_attrs", (DL_FUNC) &_H5Oget_num_attrs, 1},
   //  {"_H5Oget_info", (DL_FUNC) &_H5Oget_info, 1},
   //  {"_H5Oget_info_by_name", (DL_FUNC) &_H5Oget_info_by_name, 2},
@@ -99,34 +100,44 @@ static R_CallMethodDef libraryRCalls[] = {
   {"_H5Sget_simple_extent_dims", (DL_FUNC) &_H5Sget_simple_extent_dims, 1},
   {"_H5Sset_extent_simple", (DL_FUNC) &_H5Sset_extent_simple, 3},
   {"_H5Sget_select_npoints", (DL_FUNC) &_H5Sget_select_npoints, 1},
+  {"_H5Sget_select_hyper_nblocks", (DL_FUNC) &_H5Sget_select_hyper_nblocks, 1},
+  {"_H5Sget_select_hyper_blocklist", (DL_FUNC) &_H5Sget_select_hyper_blocklist, 4},
+  {"_H5Sselect_all", (DL_FUNC) &_H5Sselect_all, 1},
   {"_H5Sselect_none", (DL_FUNC) &_H5Sselect_none, 1},
+  {"_H5Sselect_valid", (DL_FUNC) &_H5Sselect_valid, 1},
+  {"_H5Sselect_elements", (DL_FUNC) &_H5Sselect_elements, 4},
   {"_H5Sselect_hyperslab", (DL_FUNC) &_H5Sselect_hyperslab, 6},
+  {"_H5Scombine_hyperslab", (DL_FUNC) &_H5Scombine_hyperslab, 6},
+  {"_H5Scombine_select", (DL_FUNC) &_H5Scombine_select, 3},
   {"_H5Sselect_index", (DL_FUNC) &_H5Sselect_index, 3},
   {"_H5Tcopy", (DL_FUNC) &_H5Tcopy, 1},
   {"_H5Tset_size", (DL_FUNC) &_H5Tset_size, 2},
   {"_H5Tget_size", (DL_FUNC) &_H5Tget_size, 1},
   {"_H5Tset_strpad", (DL_FUNC) &_H5Tset_strpad, 2},
   {"_H5Tget_strpad", (DL_FUNC) &_H5Tget_strpad, 1},
+  {"_H5Tset_cset", (DL_FUNC) &_H5Tset_cset, 2},
+  {"_H5Tget_cset", (DL_FUNC) &_H5Tget_cset, 1},
+  {"_H5Tis_variable_str", (DL_FUNC) &_H5Tis_variable_str, 1},
   {"_H5Pcreate", (DL_FUNC) &_H5Pcreate, 1},
   {"_H5Pget_class", (DL_FUNC) &_H5Pget_class, 1},
   {"_H5Pcopy", (DL_FUNC) &_H5Pcopy, 1},
   {"_H5Pclose", (DL_FUNC) &_H5Pclose, 1},
   {"_H5Pclose_class", (DL_FUNC) &_H5Pclose_class, 1},
-  /* {"_H5Pget_version", (DL_FUNC) &_H5Pget_version, 5}, */
-  /* {"_H5Pset_userblock", (DL_FUNC) &_H5Pset_userblock, 2}, */
-  /* {"_H5Pget_userblock", (DL_FUNC) &_H5Pget_userblock, 2}, */
-  /* {"_H5Pset_sizes", (DL_FUNC) &_H5Pset_sizes, 3}, */
-  /* {"_H5Pget_sizes", (DL_FUNC) &_H5Pget_sizes, 3}, */
-  /* {"_H5Pset_sym_k", (DL_FUNC) &_H5Pset_sym_k, 3}, */
-  /* {"_H5Pget_sym_k", (DL_FUNC) &_H5Pget_sym_k, 3}, */
-  /* {"_H5Pset_istore_k", (DL_FUNC) &_H5Pset_istore_k, 2}, */
-  /* {"_H5Pget_istore_k", (DL_FUNC) &_H5Pget_istore_k, 2}, */
-  /* {"_H5Pset_shared_mesg_nindexes", (DL_FUNC) &_H5Pset_shared_mesg_nindexes, 2}, */
-  /* {"_H5Pget_shared_mesg_nindexes", (DL_FUNC) &_H5Pget_shared_mesg_nindexes, 2}, */
-  /* {"_H5Pset_shared_mesg_index", (DL_FUNC) &_H5Pset_shared_mesg_index, 4}, */
-  /* {"_H5Pget_shared_mesg_index", (DL_FUNC) &_H5Pget_shared_mesg_index, 4}, */
-  /* {"_H5Pset_shared_mesg_phase_change", (DL_FUNC) &_H5Pset_shared_mesg_phase_change, 3}, */
-  /* {"_H5Pget_shared_mesg_phase_change", (DL_FUNC) &_H5Pget_shared_mesg_phase_change, 3}, */
+  {"_H5Pget_version", (DL_FUNC) &_H5Pget_version, 1},
+  {"_H5Pset_userblock", (DL_FUNC) &_H5Pset_userblock, 2},
+  {"_H5Pget_userblock", (DL_FUNC) &_H5Pget_userblock, 1},
+  {"_H5Pset_sizes", (DL_FUNC) &_H5Pset_sizes, 3},
+  {"_H5Pget_sizes", (DL_FUNC) &_H5Pget_sizes, 1},
+  {"_H5Pset_sym_k", (DL_FUNC) &_H5Pset_sym_k, 3},
+  {"_H5Pget_sym_k", (DL_FUNC) &_H5Pget_sym_k, 1},
+  {"_H5Pset_istore_k", (DL_FUNC) &_H5Pset_istore_k, 2},
+  {"_H5Pget_istore_k", (DL_FUNC) &_H5Pget_istore_k, 1},
+  {"_H5Pset_shared_mesg_nindexes", (DL_FUNC) &_H5Pset_shared_mesg_nindexes, 2},
+  {"_H5Pget_shared_mesg_nindexes", (DL_FUNC) &_H5Pget_shared_mesg_nindexes, 1},
+  {"_H5Pset_shared_mesg_index", (DL_FUNC) &_H5Pset_shared_mesg_index, 4},
+  {"_H5Pget_shared_mesg_index", (DL_FUNC) &_H5Pget_shared_mesg_index, 2},
+  {"_H5Pset_shared_mesg_phase_change", (DL_FUNC) &_H5Pset_shared_mesg_phase_change, 3},
+  {"_H5Pget_shared_mesg_phase_change", (DL_FUNC) &_H5Pget_shared_mesg_phase_change, 1},
   /* {"_H5Pset_driver", (DL_FUNC) &_H5Pset_driver, 3}, */
   /* {"_H5Pget_driver", (DL_FUNC) &_H5Pget_driver, 1}, */
   /* {"_H5Pget_driver_info", (DL_FUNC) &_H5Pget_driver_info, 1}, */
@@ -304,11 +315,9 @@ static R_CallMethodDef libraryRCalls[] = {
   {"_h5createDataFrame", (DL_FUNC) &_h5createDataFrame, 5},
   {"_h5errorHandling", (DL_FUNC) &_h5errorHandling, 1},
   {"_h5fileLock", (DL_FUNC) &_h5fileLock, 1},
-  {"_H5Sselect_cols", (DL_FUNC) &_H5Sselect_cols, 5},
   {"_h5listOpenObjects", (DL_FUNC) &_h5listOpenObjects, 1},
 #ifdef _H5P_filters
-  //{"_H5Pset_lzf", (DL_FUNC) &_H5Pset_lzf, 1},
-  //{"_H5Pset_lz4", (DL_FUNC) &_H5Pset_lz4, 1},
+  {"_H5Pset_lzf", (DL_FUNC) &_H5Pset_lzf, 2},
   {"_H5Pset_bzip2", (DL_FUNC) &_H5Pset_bzip2, 2},
   {"_H5Pset_blosc", (DL_FUNC) &_H5Pset_blosc, 6},
 #endif
