@@ -7,9 +7,6 @@ HDF5_conf_file <- system.file("include/H5pubconf.h", package  = "Rhdf5lib")
 if(file.exists(HDF5_conf_file)) {
     run_test <- any(grepl(x = readLines(HDF5_conf_file), "H5_HAVE_ROS3_VFD[[:blank:]]+1"))
 } 
-if(.Platform$OS.type == "windows") {
-    run_test <- FALSE
-}
 
 if(run_test) {
 
@@ -18,12 +15,18 @@ if(run_test) {
     public_S3_url <- "https://rhdf5-public.s3.eu-central-1.amazonaws.com/h5ex_t_array.h5"
     
     test_that("h5ls() works for files in public S3 buckets", {
+        
+        skip_if_offline('rhdf5-public.s3.eu-central-1.amazonaws.com')
+        
         expect_silent(h5ls_out <- h5ls(public_S3_url, s3 = TRUE))
         expect_is(h5ls_out, "data.frame")
         expect_true( "DS1" %in% h5ls_out$name )
     })
     
     test_that("h5dump() works for files in public S3 buckets", {
+        
+        skip_if_offline('rhdf5-public.s3.eu-central-1.amazonaws.com')
+        
         expect_silent(h5dump_out <- h5dump(public_S3_url, s3 = TRUE))
         expect_is( h5dump_out, "list" )
         expect_equivalent( length(h5dump_out), 1 )
