@@ -81,13 +81,38 @@ test_that("Create more datasets with different data types", {
     expect_is(contents$char[1,1], "character")
 })
 
+test_that("datasets of fixed and variable length characters can be created", {
+  
+  expect_true(
+    h5createDataset(file = h5File, dataset = "variable_length", 
+                    dims = 1, storage.mode = "character", size = NULL)
+  )
+  expect_true(
+    h5createDataset(file = h5File, dataset = "fixed_length", dims = 1, 
+                    storage.mode = "character", size = 10)
+  )
+  
+  fid <- H5Fopen(h5File)
+  did_var <- H5Dopen(fid, name = "variable_length")
+  tid_var <- H5Dget_type(did_var)
+  did_fix <- H5Dopen(fid, name = "fixed_length")
+  tid_fix <- H5Dget_type(did_fix)
+  
+  ## Variable length strings return NULL when queried for size
+  expect_true(H5Tis_variable_str(tid_var))
+  expect_false(H5Tis_variable_str(tid_fix))
+  expect_identical(H5Tget_size(tid_fix), 10L)
+  
+  sapply(c(did_var, did_fix), FUN = H5Dclose)
+  H5Fclose(fid)
+})
+  
+
 test_that("Invalid storage mode arguments", {
     expect_error( h5createDataset(file = h5File, dataset = "foo", dims = c(1,1), storage.mode = "foo") )
     expect_error( h5createDataset(file = h5File, dataset = "foo", dims = c(1,1), storage.mode = 10) ) 
     expect_error( h5createDataset(file = h5File, dataset = "foo", dims = c(1,1), storage.mode = 1L) )
     expect_error( h5createDataset(file = h5File, dataset = "foo", dims = c(1,1), storage.mode = FALSE) )
-    ## character without size argument
-    expect_error( h5createDataset(file = h5File, dataset = "foo", dims = c(1,1), storage.mode = "character", size = NULL) )
 })
 
 
@@ -128,7 +153,7 @@ test_that("Extendible datasets", {
 
     h5set_extent(h5File, "extend", c(7,3))
     h5write(mtx3x3, file = h5File, name = "extend",
-            start = c(5,1))
+            index = list(5:7, 1:3))
     expect_equal(h5read(h5File, "extend"),
                  rbind(mtx4x3, mtx3x3))
 
@@ -138,7 +163,7 @@ test_that("Extendible datasets", {
     ## [ mtx3x3 mtx7x2 ]
     h5set_extent(h5File, "extend", c(7,5))
     h5write(mtx7x2, file = h5File, name = "extend",
-            start = c(1,4))
+            index = list(1:7, 4:5))
     expect_equal(h5read(h5File, "extend"),
                  cbind(rbind(mtx4x3, mtx3x3),
                        mtx7x2))
@@ -151,17 +176,17 @@ test_that("Extendible datasets", {
     h5write( mtx4x3, file = h5File, name = "nonCompressed")
     h5set_extent(h5File, "nonCompressed", c(7,3))
     h5write(mtx3x3, file = h5File, name = "nonCompressed",
-            start = c(5,1))
+            index = list(5:7, 1:3))
     expect_equal(h5read(h5File, "nonCompressed"),
                  rbind(mtx4x3, mtx3x3))
 
 })
 
 test_that("Invalid inputs", {
-    expect_message(suppressWarnings(
-      h5createDataset(file = h5File, dataset = "fail", dims = "twenty"))
-      ) %>%
-      expect_false()
+    expect_error(
+      suppressWarnings( h5createDataset(file = h5File, dataset = "fail", dims = "twenty") ), 
+      regexp = "Can not create dataset. 'dims' and 'maxdims' must be numeric"
+    ) 
     expect_message(h5createDataset(file = h5File, dataset = "A", dims = c(20, 10))) %>%
       expect_false()
     expect_error(h5createDataset(file = h5File, dataset = "fail", dims = c(-10, 20)))
@@ -171,6 +196,10 @@ test_that("Invalid inputs", {
     expect_error(h5createDataset(file = h5File, dataset = "fail", dims = c(10, 20), maxdims = c(20, 10)))
     expect_warning(h5createDataset(file = h5File, dataset = "fail", dims = c(10, 20), level = 1, chunk = NULL))
     
+    expect_warning(h5createDataset(file = h5File, dataset = "chunkTooLarge",
+                                   dims = c(10,20), chunk = c(10,50)), 
+                   regexp = "One or more chunk dimensions exceeded the maximum for the dataset")
+    
 })
 
 test_that("Using chunks greater than 4GB", {
@@ -217,8 +246,74 @@ test_that("Invalid storage mode arguments", {
     expect_error( h5createAttribute(file = h5File, obj = "foo", dims = c(1,1), attr = "bad_attr", storage.mode = 10) ) 
     expect_error( h5createAttribute(file = h5File, obj = "foo", dims = c(1,1), attr = "bad_attr", storage.mode = 1L) )
     expect_error( h5createAttribute(file = h5File, obj = "foo", dims = c(1,1), attr = "bad_attr", storage.mode = FALSE) )
-    ## character without size argument
-    expect_error( h5createAttribute(file = h5File, obj = "foo", dims = c(1,1), attr = "bad_attr", storage.mode = "character", size = NULL) )
+})
+
+test_that("variable string attributes are created correctly", {
+    h5createAttribute(file = h5File, obj = "foo", dims = c(1,1), attr = "var_str_attr", 
+                      storage.mode = "character", size = NULL)
+    
+    fhandle <- H5Fopen(h5File)
+    dhandle <- H5Dopen(fhandle, "foo")
+    ahandle <- H5Aopen(dhandle, "var_str_attr")
+    dtype <- H5Aget_type(ahandle)
+    expect_null(H5Tget_size(dtype)) # NULL = variable length.
+    h5closeAll()
+
+    # For comparison.
+    h5createAttribute(file = h5File, obj = "foo", dims = c(1,1), attr = "var_str_attr2", 
+                      storage.mode = "character", size = 10)
+
+    fhandle <- H5Fopen(h5File)
+    dhandle <- H5Dopen(fhandle, "foo")
+    ahandle <- H5Aopen(dhandle, "var_str_attr2")
+    dtype <- H5Aget_type(ahandle)
+    expect_identical(H5Tget_size(dtype), 10L) 
+    h5closeAll()
+})
+
+test_that("string encoding is handled properly", {
+    h5createAttribute(file = h5File, obj = "foo", dims = c(1,1), attr = "ascii_str_attr", 
+                      storage.mode = "character", size = NULL)
+    
+    fhandle <- H5Fopen(h5File)
+    dhandle <- H5Dopen(fhandle, "foo")
+    ahandle <- H5Aopen(dhandle, "ascii_str_attr")
+    dtype <- H5Aget_type(ahandle)
+    expect_identical(H5Tget_cset(dtype), 0L) # aka ASCII
+    h5closeAll()
+
+    # Now Unicode.
+    h5createAttribute(file = h5File, obj = "foo", dims = c(1,1), attr = "utf_str_attr", 
+                      storage.mode = "character", cset="UTF8", size = NULL)
+    
+    fhandle <- H5Fopen(h5File)
+    dhandle <- H5Dopen(fhandle, "foo")
+    ahandle <- H5Aopen(dhandle, "utf_str_attr")
+    dtype <- H5Aget_type(ahandle)
+    expect_identical(H5Tget_cset(dtype), 1L) # aka UTF8
+    h5closeAll()
+})
+
+test_that("scalar dataspaces are created properly", {
+    # For reference.
+    h5createAttribute(file = h5File, obj = "foo", dims = 1, attr = "simple_int_attr", 
+                      storage.mode = "integer")
+    fhandle <- H5Fopen(h5File)
+    dhandle <- H5Dopen(fhandle, "foo")
+    ahandle <- H5Aopen(dhandle, "simple_int_attr")
+    space <- H5Aget_space(ahandle)
+    expect_identical(H5Sget_simple_extent_dims(space)$rank, 1L)
+    h5closeAll()
+
+    # Now, creating the scalar space.
+    h5createAttribute(file = h5File, obj = "foo", dims = NULL, attr = "scalar_int_attr", 
+                      storage.mode = "integer")
+    fhandle <- H5Fopen(h5File)
+    dhandle <- H5Dopen(fhandle, "foo")
+    ahandle <- H5Aopen(dhandle, "scalar_int_attr")
+    space <- H5Aget_space(ahandle)
+    expect_identical(H5Sget_simple_extent_dims(space)$rank, 0L)
+    h5closeAll()
 })
 
 test_that("No open HDF5 objects are left", {
